/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package cz.muni.fi.cepportal.cepserver.serviceprovider;

import com.espertech.esper.client.ConfigurationEventTypeXMLDOM;
import com.espertech.esper.client.ConfigurationException;
import com.espertech.esper.client.EPException;
import com.espertech.esper.client.EPServiceProvider;
import com.espertech.esper.client.EPStatement;
import com.espertech.esper.client.EPStatementException;
import com.espertech.esper.client.EPStatementSyntaxException;
import com.espertech.esper.client.EventBean;
import com.espertech.esper.client.PropertyAccessException;
import com.espertech.esper.client.UpdateListener;
import cz.muni.fi.cepportal.cepserver.ServerResponseMessager;
import java.io.IOException;
import java.io.Serializable;
import java.util.Set;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * MBEAN class which realize all client operations called via JMX.
 * @author Štefan Repček
 */
public class EsperServiceProvider implements EsperServiceProviderMBean, Serializable {

    private static Log log = LogFactory.getLog(EsperServiceProvider.class);
    private EPServiceProvider engine;
    private ServerResponseMessager responseMessager;

    public EsperServiceProvider(EPServiceProvider engine, ServerResponseMessager responseMessager) {
        if (engine == null) {
            throw new IllegalArgumentException("No engine instance");
        }
        this.engine = engine;
        this.responseMessager = responseMessager;
    }

    @Override
    public void createEPL(String expression, String statementName, String listener, boolean isPattern) throws IOException {
        try {
            EPStatement stmt;
            if (isPattern) {
                stmt = engine.getEPAdministrator().createPattern(expression, statementName);
                log.info("Created pattern '" + expression + "' named '" + statementName + "'");
                responseMessager.sendResponse("Created pattern '" + expression + "' named '" + statementName + "'");
            } else {

                stmt = engine.getEPAdministrator().createEPL(expression, statementName);
                log.info("Created statement '" + expression + "' named '" + statementName + "'");
                responseMessager.sendResponse("Created statement '" + expression + "' named '" + statementName + "'");

            }

            if (listener != null && !"".equals(listener)) {
                String[] rows = listener.split("@");
                int size = rows.length;
                String[][] listenerProperties = new String[2][size];
                for (int i = 0; i < size; i++) {
                    String[] pair = rows[i].split("&");
                    listenerProperties[0][i] = pair[0];
                    listenerProperties[1][i] = pair[1];
                }
                NewListener eplListener = new NewListener(listenerProperties);
                stmt.addListener(eplListener);
                log.info("listener added");
            }


        } catch (EPStatementSyntaxException ex) {
            log.error(ex);
            responseMessager.sendResponse("Your Statement contains Syntax Error: " + ex.getMessage());
        } catch (EPStatementException ex) {
            log.error(ex);
            responseMessager.sendResponse(ex.getMessage());
        }
     

    }

    @Override
    public void destroyAllStatements() throws IOException {
        engine.getEPAdministrator().destroyAllStatements();

        responseMessager.sendResponse("All Statements destroyed");

    }

    @Override
    public void restartEngine() throws IOException {
        engine.initialize();
        responseMessager.sendResponse("Engine cleaned");

    }

    /**
     *
     * @param statementName
     */
    @Override
    public void destroyEPL(String statementName) throws IOException {
        EPStatement statement = engine.getEPAdministrator().getStatement(statementName);
        if (statement != null) {
            statement.destroy();
            log.info("Statement destroyed '" + statementName + "'");
            responseMessager.sendResponse("Statement destroyed with name='" + statementName + "'");
        } else {
            log.error("such statement does not exist");
            responseMessager.sendResponse("Statement " + statementName + " does not exist.");
        }


    }

    @Override
    public void registerEventType(String xsdScheme, String rootElementName, String eventName) throws IOException {


        try {

            if (!engine.getEPAdministrator().getConfiguration().isEventTypeExists(eventName)) {
                ConfigurationEventTypeXMLDOM event = new ConfigurationEventTypeXMLDOM();
                event.setRootElementName(rootElementName);
                event.setSchemaText(xsdScheme);



                engine.getEPAdministrator().getConfiguration().addEventType(eventName, event);



                log.info("xml event type '" + eventName + "' sucessfuly registered");

                responseMessager.sendResponse("xml event type '" + eventName + "' sucessfuly registered");
            } else {
                log.info("ERROR: xml event type with name '" + eventName + "' already exist");
                responseMessager.sendResponse("ERROR: xml event type with name '" + eventName + "' already exist");
            }


        } catch (ConfigurationException ex) {
            log.error(ex);
            responseMessager.sendResponse("xml schema is not valid");
        } catch (EPException ex) {
            log.error(ex);
            responseMessager.sendResponse(ex.getMessage());

        }
    }

    @Override
    public void removeEventType(String eventName) throws IOException {
        if (engine.getEPAdministrator().getConfiguration().isEventTypeExists(eventName)) {
            Set<String> usedStatement = engine.getEPAdministrator().getConfiguration().getEventTypeNameUsedBy(eventName);
            for (String statementName : usedStatement) {
                EPStatement statement = engine.getEPAdministrator().getStatement(statementName);
                statement.destroy();
            }
            engine.getEPAdministrator().getConfiguration().removeEventType(eventName, true);

            log.info("xml event type with name '" + eventName + "' was removed");
            responseMessager.sendResponse("xml event type with name '" + eventName + "' was removed");
        } else {
            responseMessager.sendResponse("xml event type with name '" + eventName + "' does not exist");
        }
    }
    /*
     *  represents Listener for Statements
     * 
     */
    private class NewListener implements UpdateListener {

        private String[][] listenerProperties;

        public NewListener(String[][] listenerProperties) {
            this.listenerProperties = listenerProperties;
        }

        @Override
        public void update(EventBean[] newEvents, EventBean[] oldEvents) {

            try {
                int rowsLength;
                if(oldEvents== null)
                {
                    rowsLength = newEvents.length;
                }
                else if(newEvents== null)
                {
                    rowsLength = oldEvents.length;
                }
                else
                {
                    rowsLength = (newEvents.length > oldEvents.length) ? newEvents.length  : oldEvents.length;
                }
                int length = listenerProperties[0].length;
                String output = new String();
                
                String[] rows = new String[rowsLength];
                for(int i = 0;i < rows.length; i++)
                {
                    rows[i] = new String();
                }
                for (int i = 0; i < length; i++) {
                    String oldLabel = new String();
                    String newLabel = new String();
                    if (!listenerProperties[0][i].equals("")) {
                        if (listenerProperties[0][i].charAt(0) == '§') {
                            
                            if (oldEvents != null) {
                                oldLabel = listenerProperties[0][i].replace("§", "");
                          
                            }
                        } else {
                            if (newEvents != null) {
                                  newLabel = listenerProperties[0][i];
                 
                            }
                        }
                    }
                    if (listenerProperties[1][i].charAt(0) == '§') {
                        String oldExpression = listenerProperties[1][i].replace("§", "");
                        if (oldEvents != null) {
                            String[] pair = oldExpression.split("~");
                               if (pair[0].equals("e")) {
                                int j = 0;
                                for (EventBean e : oldEvents) {
                                    String expression = new String();                       
                                    if (!"".equals(pair[1])) {
                                        Object oldValue = e.get(pair[1]);
                                        if (oldValue == null) {
                                            return; // because of Esper win:time and win:batch_time bug, which invoke listener two times, second time is value null
                                        }
                                        expression = oldValue.toString();

                                    }
                                
                                    rows[j] = rows[j].concat(oldLabel + " " + expression + " ");
                                    j++;

                                }
                            } else {
                                 int targetRow = Integer.valueOf(pair[0]);
                            String expression = new String();
                            if (!"".equals(pair[1])) {
                                Object oldValue = oldEvents[targetRow].get(pair[1]);

                                if (oldValue == null) {
                                    return; // because of Esper win:time and win:batch_time bug, which invoke listener two times, second time is value null
                                }

                                expression = oldValue.toString();
                   
                            }
                        
                            rows[targetRow] = rows[targetRow].concat(oldLabel + " " + expression + " ");
                        }

                        }
                    } else {
                        if (newEvents != null) {
                            String[] pair = listenerProperties[1][i].split("~", -1);
                            if (pair[0].equals("e")) {
                                int j = 0;
                                for (EventBean e : newEvents) {
                                    String expression = new String();
       
                                    if (!"".equals(pair[1])) {
                                        Object newValue = e.get(pair[1]);
                                        if (newValue == null) {
                                            return; // because of Esper win:time and win:batch_time bug, which invoke listener two times, second time is value null
                                        }
                                        expression = newValue.toString();

                                    }
                                  
                                    rows[j] = rows[j].concat(newLabel + " " + expression + " ");
                                    j++;

                                }
                            } else {
                                int targetRow = Integer.valueOf(pair[0]);
                                String expression = new String();
                                if (!"".equals(pair[1])) {
                                    Object newValue = newEvents[targetRow].get(pair[1]);

                                    if (newValue == null) {
                                        return; // because of Esper win:time and win:batch_time bug, which invoke listener two times, second time is value null
                                    }
                                    expression = newValue.toString();

                                  
                                }
                            
                                rows[targetRow] = rows[targetRow].concat(newLabel + " " + expression + " ");
                            }

                        }



                    }
                }
                for (String s : rows) {
                    output = output.concat(s);
                }
                if(!output.equals(""))
                {
                responseMessager.sendResponse(output);
                }
                log.info(output);
            } catch (PropertyAccessException ex) {
                responseMessager.sendResponse("Listener ERROR:" + ex.getMessage());
            }
              catch (NumberFormatException ex) {
                responseMessager.sendResponse("Listener ERROR: wrong index format");
            }
        }
    }
}
