/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package cz.muni.fi.cepportal.backend;


import cz.muni.fi.cepportal.jms.JMSContext;
import cz.muni.fi.cepportal.jms.JMSContextFactory;
import java.io.IOException;
import java.io.InputStream;
import java.io.Serializable;
import java.net.ConnectException;
import java.util.Properties;
import java.util.Timer;
import java.util.TimerTask;
import javax.faces.application.FacesMessage;
import javax.faces.bean.ApplicationScoped;
import javax.faces.bean.ManagedBean;
import javax.faces.context.FacesContext;
import javax.jms.DeliveryMode;
import javax.jms.JMSException;
import javax.jms.MessageConsumer;
import javax.jms.MessageProducer;
import javax.jms.TextMessage;
import javax.management.MBeanServerConnection;
import javax.management.MBeanServerInvocationHandler;
import javax.management.MalformedObjectNameException;
import javax.management.ObjectName;
import javax.management.remote.JMXConnector;
import javax.management.remote.JMXConnectorFactory;
import javax.management.remote.JMXServiceURL;
import javax.naming.NamingException;
import javax.naming.ServiceUnavailableException;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.Text;

/**
 * Managed Bean fo remote comuniacation, Methods of this class sends user
 * requests to engine Esper All operations are realised via
 * EsperServcicePoviderMBean
 *
 * @author Štefan Repček
 */
@ManagedBean
@ApplicationScoped // because of session timeout
public class ClientManagedBean implements Serializable {

    private static Log log = LogFactory.getLog(ClientManagedBean.class);
    private EsperServiceProviderMBean serviceProvider;
    private ResponseMessageListener messageListener;
    private String serverIP = "localhost";

    public String getServerIP() {
        return serverIP;
    }

    public void setServerIP(String serverIP) {
        this.serverIP = serverIP;
    }

    public ResponseMessageListener getMessageListener() {
        return messageListener;
    }
    private JMSContext jmsCtx;
    private Properties properties;

    /*
     *  This method connects client to server
     */
    public void establishConnection() throws IOException, MalformedObjectNameException, JMSException, NamingException {
        log.info("Loading properties");
        properties = new Properties();
        InputStream propertiesIS = ClientManagedBean.class.getClassLoader().getResourceAsStream(ClientConstants.CONFIG_FILENAME);
        if (propertiesIS == null) {
            throw new RuntimeException("Properties file '" + ClientConstants.CONFIG_FILENAME + "' not found in classpath");
        }
        properties.load(propertiesIS);

        if (tryAttachToServerViaJMX() && connectToJMSServer()) {


            FacesContext.getCurrentInstance().getExternalContext().redirect("eplstatements.xhtml");

        }




    }
    /*
     *  This method creates and starts EPL or pattern Statement on engine Esper
     * 
     */

    public void createEPL(String expression, String statementName, String listener, boolean isPattern) throws IOException {
        if (BackendHelpers.validateEPLStatement(expression, statementName)) {
            try {
                serviceProvider.createEPL(expression, statementName, listener, isPattern);
            } catch (IOException ex) {
                boolean serverOnline = tryAttachToServerViaJMX(); // if server not available try reconnect once (because when is server restarted client throw IO Exception)
                if (serverOnline) {
                    serviceProvider.createEPL(expression, statementName, listener, isPattern);
                } else {
                    log.error(ex);
                }

            }
        }

    }

    /*
     *  This method stops and destroys EPL or pattern Statement in engine Esper
     * 
     */
    public void destroyEPL(String statementName) throws IOException {
        try {
            serviceProvider.destroyEPL(statementName);
        } catch (IOException ex) { // if server not available try reconnect once (because when is server restarted client throw IO Exception) 
            boolean serverOnline = tryAttachToServerViaJMX();
            if (serverOnline) {
                serviceProvider.destroyEPL(statementName);
            } else {
                log.error(ex);
            }
        }
    }

    /*
     *  This method restart engine Esper
     * 
     */
    public void restartEngine() throws IOException {
        try {
            serviceProvider.restartEngine();
        } catch (IOException ex) {  // if server not available try reconnect once (because when is server restarted client throw IO Exception)
            boolean serverOnline = tryAttachToServerViaJMX();
            if (serverOnline) {
                serviceProvider.restartEngine();
            } else {
                log.error(ex);
            }
        }
    }
    /*
     *  This method destroys all statements in engine Esper
     */

    public void destroyAllStatements() throws IOException {
        try {
            serviceProvider.destroyAllStatements();
        } catch (IOException ex) {  // if server not available try reconnect once (because when is server restarted client throw IO Exception)
            boolean serverOnline = tryAttachToServerViaJMX();
            if (serverOnline) {
                serviceProvider.destroyAllStatements();
            } else {
                log.error(ex);
            }

        }
    }
    /*
     *  This method register event in engine Esper
     */

    public void registerEventType(String scheme, String rootElementName, String name) throws IOException {
        if (BackendHelpers.validateEventType(scheme, rootElementName, name)) {
            try {
                serviceProvider.registerEventType(scheme, rootElementName, name);
            } catch (IOException ex) {  // if server not available try reconnect once (because when is server restarted client throw IO Exception)
                boolean serverOnline = tryAttachToServerViaJMX();
                if (serverOnline) {
                    serviceProvider.registerEventType(scheme, rootElementName, name);
                } else {
                    log.error(ex);
                }
            }
        }


    }
    /*
     *  This method unregister event in engine Esper
     */

    public void removeEventType(String eventName) throws IOException {

        try {
            serviceProvider.removeEventType(eventName);
        } catch (IOException ex) {   // if server not available try reconnect once (because when is server restarted client throw IO Exception)
            boolean serverOnline = tryAttachToServerViaJMX();
            if (serverOnline) {
                serviceProvider.removeEventType(eventName);
            } else {
                log.error(ex);
            }

        }

    }

    /*
     *  This method starts event stream
     */
    public void sendEvents(Document xmlEvents) throws JMSException, InterruptedException {
        if (xmlEvents != null) {
            MessageProducer producer = jmsCtx.getSession().createProducer(jmsCtx.getServerDestination());


            try {

                DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
                factory.setIgnoringElementContentWhitespace(true);
                DocumentBuilder builder;
                builder = factory.newDocumentBuilder();


                Node root = xmlEvents.getFirstChild();
                NodeList childs = root.getChildNodes();
                Timer timer = new Timer();
                int time = 0;
                for (int i = 0; i < childs.getLength(); i++) {
                    Document event = builder.newDocument();
                    SendEventTask eventTask = new SendEventTask(childs.item(i), producer, event);
                    if (!(childs.item(i) instanceof Text)) {
                        if (childs.item(i).getNodeName().equals("time")) {
                            time += Integer.valueOf(childs.item(i).getTextContent());
                        } else {
                            timer.schedule(eventTask, time);
                        }
                    }
                }


            } catch (NumberFormatException ex) {
                log.error(ex);
                FacesContext context = FacesContext.getCurrentInstance();
                context.addMessage("event_stream", new FacesMessage("ERROR: wrong number format of time in event stream xml file"));
            } catch (ParserConfigurationException ex) {
                log.error(ex);
            }
        } else {

            BackendHelpers.sendFacesMessage("event_stream", "Event stream XML file can not be empty");
        }


    }

    private class SendEventTask extends TimerTask {

        public SendEventTask(Node node, MessageProducer producer, Document event) {
            this.node = node;
            this.producer = producer;
            this.event = event;

        }
        private Node node;
        private MessageProducer producer;
        private Document event;

        @Override
        public void run() {
            try {
                Element imported = (Element) event.importNode(node, Boolean.TRUE);
                event.appendChild(imported);
                String stringEvent = BackendHelpers.xmlDocToString(event);

                TextMessage message = jmsCtx.getSession().createTextMessage();
                message.setText(stringEvent);
                message.setJMSDeliveryMode(DeliveryMode.NON_PERSISTENT);
                producer.send(message);
            } catch (JMSException ex) {
                log.error(ex);
            }
        }
    }
    /*
     *  This method attach client to server via JMX
     * 
     */

    private boolean tryAttachToServerViaJMX() throws IOException {
        try {
            JMXServiceURL url = new JMXServiceURL("service:jmx:rmi:///jndi/rmi://" + serverIP + ":1199/server");
            log.info("Attach to server via JMX " + url);
            JMXConnector jmxc = JMXConnectorFactory.connect(url, null);
            MBeanServerConnection mbsc = jmxc.getMBeanServerConnection();
            ObjectName mBeanName = new ObjectName(ClientConstants.MBEAN_NAME);
            serviceProvider = (EsperServiceProviderMBean) MBeanServerInvocationHandler.newProxyInstance(
                    mbsc, mBeanName, EsperServiceProviderMBean.class, true);

        } catch (MalformedObjectNameException ex) {
            log.error(ex);
            return false;
        } catch (IOException ex) {
            if (ex.getCause() instanceof ServiceUnavailableException) {

                FacesContext.getCurrentInstance().getExternalContext().getFlash().setKeepMessages(true);
                FacesContext context = FacesContext.getCurrentInstance();
                context.addMessage("preface", new FacesMessage("ERROR: Client can not connect to server"));
                FacesContext.getCurrentInstance().getExternalContext().redirect("index.xhtml");


                log.error(ex);


            }
            return false;
        }
        return true;



    }
    /*
     *  This method connect client to JMS provider
     *  
     */

    private boolean connectToJMSServer() throws JMSException, NamingException {
        try {
            String factory = properties.getProperty(ClientConstants.JMS_CONTEXT_FACTORY);
            String jmsProviderPort = properties.getProperty(ClientConstants.JMS_PROVIDER_PORT);
            String jmsProviderURLOptions = properties.getProperty(ClientConstants.JMS_PROVIDER_URL_OPTIONS);
            String jmsurl = "tcp://" + serverIP + ":" + jmsProviderPort + jmsProviderURLOptions;
            String connFactoryName = properties.getProperty(ClientConstants.JMS_CONNECTION_FACTORY_NAME);
            String user = properties.getProperty(ClientConstants.JMS_USERNAME);
            String password = properties.getProperty(ClientConstants.JMS_PASSWORD);
            String serverDestination = properties.getProperty(ClientConstants.JMS_SERVER_DESTINATION);
            String clientDestination = properties.getProperty(ClientConstants.JMS_CLIENT_DESTINATION);
            boolean isTopic = Boolean.parseBoolean(properties.getProperty(ClientConstants.JMS_IS_TOPIC));
            jmsCtx = JMSContextFactory.createContext(factory, jmsurl, connFactoryName, user, password, serverDestination, clientDestination, isTopic);

            jmsCtx.getConnection().start();

            MessageConsumer consumer = jmsCtx.getSession().createConsumer(jmsCtx.getClientDestination());
            if (messageListener == null) {
                messageListener = new ResponseMessageListener();
            }
            consumer.setMessageListener(messageListener);
        } catch (JMSException ex) {
            if (ex.getCause() instanceof ConnectException) {
                FacesContext context = FacesContext.getCurrentInstance();
                context.addMessage("preface", new FacesMessage("ERROR: Client can not connect to ActiveMQ server"));
                log.error(ex);

            }
            return false;
        }
        return true;
    }
}
