/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package cz.muni.fi.cepportal.backend;

import cz.muni.fi.cepportal.database.dto.EventStreamDTO;
import cz.muni.fi.cepportal.database.exceptions.NonexistentEntityException;
import cz.muni.fi.cepportal.database.exceptions.PreexistingEntityException;
import cz.muni.fi.cepportal.database.services.EventStreamService;
import java.io.IOException;
import java.io.Serializable;
import java.util.List;
import java.util.Map;
import java.util.logging.Logger;
import javax.annotation.PostConstruct;
import javax.faces.application.FacesMessage;
import javax.faces.context.FacesContext;
import javax.inject.Named;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.myfaces.custom.fileupload.UploadedFile;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Scope;
import org.w3c.dom.Document;
import org.xml.sax.SAXException;

/**
 * Managed Bean for EventStream entity.  
 * 
 * @author Štefan Repček
 */
@Named("eventStreamManagedBean")
@Scope("session")
public class EventStreamManagedBean implements Serializable{
    
   @Autowired
   private EventStreamService eventStreamService;
   
   final static Log log = LogFactory.getLog(EventStreamManagedBean.class);
   
   private EventStreamDTO eventStream = new EventStreamDTO();
   private List<EventStreamDTO> eventStreams;
   private Document eventStreamXML;

    public Document getEventStreamXML() {
        return eventStreamXML;
    }

    public void setEventStreamXML(Document eventStreamXML) {
        this.eventStreamXML = eventStreamXML;
    }
   private UploadedFile eventStreamXMLfile;

    public UploadedFile getEventStreamXMLfile() {
        return eventStreamXMLfile;
    }

    public void setEventStreamXMLfile(UploadedFile eventStreamXML) {
        this.eventStreamXMLfile = eventStreamXML;
    }
    public List<EventStreamDTO> getEventStreams() {
        return eventStreams;
    }

    public void setEventStreams(List<EventStreamDTO> eventStreams) {
        this.eventStreams = eventStreams;
    }

    public EventStreamDTO getEventStream() {
        return eventStream;
    }

    public void setEventStream(EventStreamDTO eventStream) {
        this.eventStream = eventStream;
    }
    
    @PostConstruct
    public void init()
    {
        eventStreams = eventStreamService.findAllEventStreams();
    }
    
   
   
   public void add() {
        if (validate()) {
            try {
                eventStream.setId(null);
                eventStreamService.createEventStream(eventStream);
                log.info("Event Stream saved");
                init();
            } catch (PreexistingEntityException ex) {
                log.error(ex);
                BackendHelpers.sendFacesMessage("event_stream", ex.getMessage());
            }
        }

    }

    public void save() {
        if (validate()) {
            if (eventStream.getId() != null) {
                try {
                    eventStreamService.updateEventStream(eventStream);
                    log.info("Event Stream udpated");
                    init();

                } catch (PreexistingEntityException ex) {
                    log.error(ex);
                    BackendHelpers.sendFacesMessage("event_stream", ex.getMessage());
                }
            } 
        }
    }

    public void delete() throws NonexistentEntityException {
        Map<String, String> params = FacesContext.getCurrentInstance().getExternalContext().getRequestParameterMap();
        EventStreamDTO eventStreamToDelete = eventStreamService.getEventStreamByID(Long.parseLong(params.get("id")));
        eventStreamService.deleteEvent(eventStreamToDelete);
        log.info("Event Stream deleted");
        init();
    }

    public void searchByName(String name) {
        eventStreamService.findEventStreamByName(name);

    }

    public void select() {
        Map<String, String> params = FacesContext.getCurrentInstance().getExternalContext().getRequestParameterMap();
        eventStream = eventStreamService.getEventStreamByID(Long.parseLong(params.get("id")));

    }

    public void addXML() {
        if (eventStreamXMLfile != null) {
            Document xmlEventStream = BackendHelpers.fileToXMLDocument(eventStreamXMLfile);
            if (xmlEventStream != null) {
                try {
                    BackendHelpers.validateXSD(xmlEventStream);
                    
                    String stream = BackendHelpers.xmlDocToString(xmlEventStream);
                    eventStream.setEventStream(stream);
                        
                } catch (SAXException ex) {
                    log.error(ex);
                     BackendHelpers.sendFacesMessage("event_stream","event stream xml file is not valid");
                } catch (IOException ex) {
                     log.error(ex);   
                      BackendHelpers.sendFacesMessage("event_stream","event stream xml file is not valid");
                }
               
           }
       }
       
   }
   public void prepareStreamForSending()
   {  
       if(eventStreamXMLfile!=null)
       {
           eventStreamXML = BackendHelpers.fileToXMLDocument(eventStreamXMLfile);
       }
       else if(eventStream.getEventStream() != null)
       {
           eventStreamXML = BackendHelpers.xmlStringToDoc(eventStream.getEventStream());
       }
     
       
   }
   
   private boolean validate()
   {
       if(eventStream.getName().equals(""))
       {
           BackendHelpers.sendFacesMessage("event_stream","Event stream name can not be empty");
           return false;
       }
       if(eventStream.getEventStream() == null)
       {
           BackendHelpers.sendFacesMessage("event_stream","Event stream XML file can not be empty");
           return false;
       }
       return true;
   }
   
   
  
    
}
