/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package cz.muni.fi.cepportal.backend;

import cz.muni.fi.cepportal.database.dto.EventTypeDTO;
import cz.muni.fi.cepportal.database.exceptions.NonexistentEntityException;
import cz.muni.fi.cepportal.database.exceptions.PreexistingEntityException;
import cz.muni.fi.cepportal.database.services.EventTypeService;
import java.io.Serializable;
import java.util.List;
import java.util.Map;
import javax.annotation.PostConstruct;
import javax.faces.application.FacesMessage;
import javax.faces.context.FacesContext;
import javax.inject.Named;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.myfaces.custom.fileupload.UploadedFile;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Scope;
import org.w3c.dom.Document;

/**
 * Managed Bean for EventType entity. 
 * 
 * @author Štefan Repček
 */
@Named("eventTypeManagedBean")
@Scope("session")
public class EventTypeManagedBean implements Serializable {
    
   @Autowired
   private EventTypeService eventTypeService;
   
   private final static Log log = LogFactory.getLog(EventTypeManagedBean.class);
   
   private EventTypeDTO eventType = new EventTypeDTO();
   private UploadedFile eventScheme;
   private String EventTypeNameToRemove;

    public String getEventTypeNameToRemove() {
        return EventTypeNameToRemove;
    }

    public void setEventTypeNameToRemove(String EventTypeNameToRemove) {
        this.EventTypeNameToRemove = EventTypeNameToRemove;
    }

    public UploadedFile getEventScheme() {
        return eventScheme;
    }

    public void setEventScheme(UploadedFile eventScheme) {
        this.eventScheme = eventScheme;
    }
   private List<EventTypeDTO> eventTypes;

    public List<EventTypeDTO> getEventTypes() {
        return eventTypes;
    }

    public void setEventTypes(List<EventTypeDTO> events) {
        this.eventTypes = events;
    }

    public EventTypeDTO getEventType() {
        return eventType;
    }

    public void setEventType(EventTypeDTO event) {
        this.eventType= event;
    }
    
    @PostConstruct
    public void init()
    {
        eventTypes = eventTypeService.findAllEventTypes();
    }
  
    public void addScheme() {
        if (eventScheme != null) {
            Document xmlEventScheme = BackendHelpers.fileToXMLDocument(eventScheme);
            if (xmlEventScheme != null) {
                String scheme = BackendHelpers.xmlDocToString(xmlEventScheme);
                eventType.setEventScheme(scheme);
            }
        } 
    }

    public void add() {
        
        if (BackendHelpers.validateEventType(eventType.getEventScheme(), eventType.getEventRoot(), eventType.getEventName())) {
            try {
                eventType.setId(null);
                eventTypeService.createEventType(eventType);
                log.info("Event type saved");
                init();
            } catch (PreexistingEntityException ex) {
                log.error(ex);
                sendFacesMessage("event", ex.getMessage());
            }
        }
    }

    public void save() {

        if (BackendHelpers.validateEventType(eventType.getEventScheme(), eventType.getEventRoot(), eventType.getEventName())) {
            if (eventType.getId() != null) {
                try {
                    eventTypeService.updateEventType(eventType);
                    log.info("Event type udpated");
                    init();
                } catch (PreexistingEntityException ex) {
                    log.error(ex);
                    sendFacesMessage("event", ex.getMessage());
                }
            } else {
                sendFacesMessage("event", "event type " + eventType.getEventName() + " is not stored in database");
            }
        }
    }

    public void delete() throws NonexistentEntityException {
        Map<String, String> params = FacesContext.getCurrentInstance().getExternalContext().getRequestParameterMap();
        EventTypeDTO eventToDelete = eventTypeService.getEventTypeByID(Long.parseLong(params.get("id")));
        eventTypeService.deleteEventType(eventToDelete);
        log.info("Event type deleted");
        init();
    }

    public void select() {
        Map<String, String> params = FacesContext.getCurrentInstance().getExternalContext().getRequestParameterMap();
        eventType = eventTypeService.getEventTypeByID(Long.parseLong(params.get("id")));

    }

    private void sendFacesMessage(String component, String msg) {
        FacesContext context = FacesContext.getCurrentInstance();
        context.addMessage(component, new FacesMessage(msg));

    }

   
        
        
      
}
