/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package cz.muni.fi.cepportal.database.services.impl;

import cz.muni.fi.cepportal.database.dto.EPLStatementDTO;
import cz.muni.fi.cepportal.database.dto.convert.EPLStatementConverter;
import cz.muni.fi.cepportal.database.dao.EPLStatementDAO;
import cz.muni.fi.cepportal.database.entities.EPLStatement;
import cz.muni.fi.cepportal.database.exceptions.NonexistentEntityException;
import cz.muni.fi.cepportal.database.exceptions.PreexistingEntityException;
import cz.muni.fi.cepportal.database.services.EPLStatementService;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import javax.faces.bean.ManagedBean;
import javax.faces.bean.ManagedProperty;
import javax.faces.bean.NoneScoped;
import javax.inject.Inject;
import javax.inject.Named;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.dao.DataAccessException;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

/**
 * Service class which handles DAO exceptions and manages transaction
 * 
 * @author Štefan Repček
 */
@Service
public class EPLStatementServiceImpl implements EPLStatementService, Serializable{

    private static Log log = LogFactory.getLog(EPLStatementServiceImpl.class);
    
   
    @Autowired
    private EPLStatementDAO eplStatementDAO;

    public void setEplStatementDAO(EPLStatementDAO eplStatementDAO) {
        this.eplStatementDAO = eplStatementDAO;
    }
    
    
    
    @Override
    @Transactional
    public void createEPLStatement(EPLStatementDTO eplStatement) throws PreexistingEntityException {
      try
      {
        eplStatementDAO.createEPLStatement(EPLStatementConverter.dtoToEntity(eplStatement));
      }
      catch(DataAccessException ex)
      {
          log.error(ex.getMessage());
          throw ex;
      }
    }

    @Override
    @Transactional
    public void updateEPLStatement(EPLStatementDTO eplStatement) throws PreexistingEntityException {
         try
      {
        eplStatementDAO.updateEPLStatement(EPLStatementConverter.dtoToEntity(eplStatement));
      }
      catch(DataAccessException ex)
      {
          log.error(ex.getMessage());
          throw ex;
      }
    }

    @Override
    @Transactional
    public void deleteEPLStatement(EPLStatementDTO eplStatement) throws NonexistentEntityException {
       try
      {
        eplStatementDAO.deleteEPLStatement(EPLStatementConverter.dtoToEntity(eplStatement));
      }
      catch(DataAccessException ex)
      {
          log.error(ex.getMessage());
          throw ex;
      }
    }

    @Override
    @Transactional(readOnly=true)
    public EPLStatementDTO getEPLStatementbyID(Long id) {
      try
      {
        return EPLStatementConverter.entityToDTO(eplStatementDAO.getEPLStatementbyID(id));
      }
      catch(DataAccessException ex)
      {
          log.error(ex);
          throw ex;
      }
    }

    @Override
    @Transactional(readOnly=true)
    public EPLStatementDTO findEPLStatementByName(String name) {
      try
      {
        return EPLStatementConverter.entityToDTO(eplStatementDAO.findEPLStatementByName(name));
      }
      catch(DataAccessException ex)
      {
          log.error(ex);
          throw ex;
      }
    }

    @Override
    @Transactional(readOnly=true)
    public List<EPLStatementDTO> findAllEPLStatements() {
        List <EPLStatementDTO> dtos = new ArrayList<EPLStatementDTO>();
        
        for(EPLStatement s : eplStatementDAO.findAllEPLStatements())
        {
            EPLStatementDTO dto = EPLStatementConverter.entityToDTO(s);
            dtos.add(dto);
            
        }
        
        return dtos;
    }
    
}
